@@ -62,9 +62,11 @@ if __name__ == "__main__":
 
 # import custom modules
 from utils import dprint, get_user_home_dir, SingleButtonDialog, \
-                  get_treeview_selection, estimate, YesNoDialog
+                  get_treeview_selection, estimate, YesNoDialog, pretend_check
 from version import version
 
+from gettext import gettext as _
+
 # some constants for the tabs
 TAB_PROCESS = 0
 TAB_WARNING = 1
@@ -77,21 +79,22 @@ TAB_QUEUE = 4
 # you won't be able to make it stick when text is added rapidly
 SLIDER_CLOSE_ENOUGH = 0.5 # of the page size
 
-# some contant strings that may be internationalized later
-KILLED_STRING = "*** process killed ***\n"
-TERMINATED_STRING = "*** process completed ***\n"
+# some contant strings that can be internationalized
+KILLED_STRING = _("*** process killed ***\n")
+TERMINATED_STRING = _("*** process completed ***\n")
 TABS = [TAB_PROCESS, TAB_WARNING, TAB_CAUTION, TAB_INFO, TAB_QUEUE]
-TAB_LABELS = ["Process", "Warnings", "Cautions", "Summary", "Emerge queue"]
+TAB_LABELS = [_("Process"), _("Warnings"), _("Cautions"), _("Summary"), _("Emerge queue")]
 
 class ProcessManager:
     """ Manages queued and running processes """
     def __init__(self, env = {}, prefs = None, config = None, log_mode = False):
         """ Initialize """
         if log_mode:
-            self.title = "Porthole Log Viewer"
+            self.title = _("Porthole Log Viewer")
         else:
-            self.title = "Porthole-Terminal"
+            self.title = _("Porthole-Terminal")
         self.log_mode = log_mode
+        self.Semaphore = threading.Semaphore()
         # copy the environment and preferences
         self.env = env
         self.prefs = prefs
@@ -105,6 +108,8 @@ class ProcessManager:
         self.process_list = []
         # the window is not visible until a process is added
         self.window_visible = False
+	self.task_completed = True
+	self.confirm = False
         # filename and serial #
         self.directory = None
         self.filename = None
@@ -141,7 +146,7 @@ class ProcessManager:
         """ Show the process window """
         # load the glade file
         self.wtree = gtk.glade.XML(self.prefs.DATA_PATH + "porthole.glade",
-                                   "process_window")
+                                   "process_window", self.prefs.APP)
         # these need to be before the callbacks
         # setup some aliases for easier access
         self.window = self.wtree.get_widget("process_window")
@@ -174,7 +179,7 @@ class ProcessManager:
                      "on_move_up" : self.move_queue_item_up,
                      "on_move_down" : self.move_queue_item_down,
                      "on_remove" : self.remove_queue_item,
-                     "on_quit" : self.destroy_window}
+                     "on_quit" : self.menu_quit}
         self.wtree.signal_autoconnect(callbacks)
         # get a mostly blank structure to hold a number of widgets & settings
         self.term = terminal_notebook()
@@ -206,9 +211,11 @@ class ProcessManager:
         # set some persistent variables for text capture
         self.catch_seq = False
         self.escape_seq = "" # to catch the escape sequence in
+        self.first_cr = True  # first time cr is detected for a line
+        self.overwrite_till_nl = False  # overwrite until after a '\n' detected for this line
         self.resume_line = None
         # setup the queue treeview
-        column = gtk.TreeViewColumn("Packages to be merged      ")
+        column = gtk.TreeViewColumn(_("Packages to be merged      "))
         pixbuf = gtk.CellRendererPixbuf()
         column.pack_start(pixbuf, expand = False)
         column.add_attribute(pixbuf, "pixbuf", 0)
@@ -255,12 +262,16 @@ class ProcessManager:
             id = self.term.vadjustment[x].connect("value_changed", self.set_scroll)
             self.term.vhandler_id += [id]
             #self.term.auto_scroll[x] = False  # already initialized to True
+	    # create autoscroll end marks to seek to &
+	    end_mark = self.term.buffer[x].create_mark(("end_mark"+str(x)), self.term.buffer[x].get_end_iter(), False)
+	    self.term.end_mark += [end_mark]
         #dprint("TERMINAL: show_window() -- self.term.vadjustment[]," +
         #       "self.term.vhandler_id[], self.term.autoscroll")
         #dprint(self.term.vadjustment)
         #dprint(self.term.vhandler_id)
         #dprint(self.term.auto_scroll)
         self.notebook.connect("switch-page", self.switch_page)
+	self.window.connect('delete-event', self.confirm_delete)
         if self.prefs:
             self.window.resize((self.prefs.emerge.verbose and
                                 self.prefs.terminal.width_verbose or
@@ -318,15 +329,8 @@ class ProcessManager:
         dprint("TERMINAL: switch_page; page_num = %d" %page_num)
         self.term.current_tab = self.term.visible_tablist[page_num]
         if self.term.auto_scroll[self.term.current_tab]:
-            #dprint("TERMINAL: append() -- self.term.vadjustment[], self.term.vhandler_id[]")
-            #dprint(self.term.vadjustment)
-            #dprint(self.term.vhandler_id)
-            num = self.term.current_tab
-            self.term.vadjustment[num].handler_block(self.term.vhandler_id[num])
-            #result = self.term.view[num].scroll_to_iter(self.term.buffer[num].get_end_iter(),0.0, True, 0, 0.9)
-            self.term.view[num].scroll_mark_onscreen(self.term.buffer[num].get_insert())
-            self.term.vadjustment[num].handler_unblock(self.term.vhandler_id[num])
-        return
+		self.scroll_current_view()
+	return
 
     def set_scroll(self,  vadjustment):
         """Sets autoscrolling on when moved to bottom of scrollbar"""
@@ -410,6 +414,10 @@ class ProcessManager:
 
     def add_process(self, package_name, command_string, callback):
         """ Add a process to the queue """
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
+        dprint("TERMINAL: add_process; Semaphore acquired")
+
         # if it's already in the queue, don't add it!
         for data in self.process_list:
             if package_name == data[0]:
@@ -418,19 +426,25 @@ class ProcessManager:
                     if data == self.process_list[0]:
                         if self.killed:
                             # The process has been killed, so help the user out a bit
-                            message = "The package you selected is already in the emerge queue,\n" \
-                                      "but it has been killed. Would you like to resume the emerge?"
+                            message = _("The package you selected is already in the emerge queue,\n" \
+                                      "but it has been killed. Would you like to resume the emerge?")
                             result = self.resume_dialog(message)
                             if result == gtk.RESPONSE_ACCEPT: # Execute
                                 break
                             elif result == gtk.RESPONSE_YES: # Resume
                                 self.resume_normal(None)
                             else: # Cancel
+                                # Clear semaphore, we're done
+                                self.Semaphore.release()
+                                dprint("TERMINAL: add_process; Semaphore released")
                                 return gtk.FALSE
                         else:
-                            message = "The package you selected is already in the emerge queue!"
-                            SingleButtonDialog("Error Adding Package To Queue!", None,
+                            message = _("The package you selected is already in the emerge queue!")
+                            SingleButtonDialog(_("Error Adding Package To Queue!"), None,
                                            message, None, "Ok")
+                            # Clear semaphore, we're done
+                            self.Semaphore.release()
+                            dprint("TERMINAL: add_process; Semaphore released")
                             return
                 # remove process from list
                 dprint(self.process_list)
@@ -455,14 +469,19 @@ class ProcessManager:
             if not self.term.tab_showing[TAB_QUEUE]:
                 self.show_tab(TAB_QUEUE)
                 self.queue_menu.set_sensitive(gtk.TRUE)
+        # Clear semaphore, we're done
+        self.Semaphore.release()
+        dprint("TERMINAL: add_process; Semaphore released")
         # if no process is running, let's start one!
         if not self.reader.process_running:
             if self.resume_available:
                 self.start_queue(True)
+		dprint("TERMINAL: add_process; self.resume_available")
             else:
                 # pending processes, run the next one in the list
                 self.start_queue(False)
 
+
     def _run(self, command_string, iter = None):
         """ Run a given command string """
         # we can't be killed anymore
@@ -472,18 +491,7 @@ class ProcessManager:
         self.warning_count = 0
         self.caution_count = 0
         self.Failed = False
-        self.isPretend = (sre.search("--pretend", command_string) != None)
-        if not self.isPretend:
-            tmpcmdline = command_string.split()
-            #dprint(tmpcmdline)
-            for x in tmpcmdline:
-                if x[0:1]=="-"and x[1:2]!="-":
-                    for y in x[1:]:
-                        #dprint(y)    
-                        if y == "p":
-                            #dprint("found it")
-                            self.isPretend = True
-
+        self.isPretend = pretend_check(command_string)
         start_iter = self.term.buffer[TAB_PROCESS].get_end_iter()
         if self.command_start:
             # move the start mark
@@ -521,23 +529,70 @@ class ProcessManager:
                 #print e
                 os._exit(1)
 
-    def destroy_window(self, widget):
-        """ Destroy the window when the close button is pressed """
-        dprint("TERMINAL: close button clicked... destroying now")
+    def menu_quit(self, widget):
+        """ hide the window when the close button is pressed """
+	dprint("TERMINAL: menu_quit()")
+	if self.confirm_delete():
+	    return
+        dprint("TERMINAL: menu==>quit clicked... starting destruction")
         self.window.destroy()
 
     def on_process_window_destroy(self, widget, data = None):
         """Window was closed"""
+	dprint("TERMINAL: on_process_window_destroy()")
         # kill any running processes
-        self.kill()
+	self.kill_process()
         # make sure to reset the process list
+	self.Semaphore.acquire()
         self.process_list = []
+	self.Semaphore.release()
         # the window is no longer showing
         self.window_visible = False
         self.wtree = None
         if __name__ == "__main__":
             # if running standalone, quit
-            gtk.main_quit()
+	    try:
+                gtk.main_quit()
+	    except:
+		gtk.mainquit()
+	dprint("TERMINAL: on_process_window_destroy(); ...destroying now")
+	self.window.destroy
+
+    def kill_process(self, widget = None, confirm = False):
+        """ Kill currently running process """
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
+        dprint("TERMINAL: kill_process; Semaphore acquired")
+
+        if not self.reader.process_running and not self.file_input:
+            dprint("TERMINAL: No running process to kill!")
+            # We're finished, release semaphore
+            self.Semaphore.release()
+            dprint("TERMINAL: kill_process; Semaphore released")
+            return True
+	#~ if widget or confirm:
+	    #~ self.confirm = False
+	if self.kill():
+	    # We're finished, release semaphore
+            self.Semaphore.release()
+            dprint("TERMINAL: kill_process; Semaphore released")
+	    return False
+	if self.log_mode:
+            dprint("LOG: set statusbar -- log killed")
+            self.set_statusbar(_("***Log Process Killed!"))
+        else:
+            # set the queue icon to killed
+            iter = self.process_list[0][2]
+            self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_CANCEL))
+            # set the resume buttons to sensitive
+            self.set_resume(True)
+
+	dprint("TERMINAL: leaving kill_process")
+
+        # We're finished, release semaphore
+        self.Semaphore.release()
+        dprint("TERMINAL: kill_process; Semaphore released")
+        return True
 
     def kill(self):
         """Kill process."""
@@ -550,7 +605,7 @@ class ProcessManager:
             self.reader.f.close()
             self.file_input = False
             dprint("LOG: leaving kill()")
-            return
+            return True
         # If started and still running
         if self.pid and not self.killed:
             try:
@@ -566,12 +621,54 @@ class ProcessManager:
             except OSError:
                 dprint("TERMINAL: kill(), OSError")
                 pass
-            self.killed = 1
+            self.killed = True
             if self.term.tab_showing[TAB_QUEUE]:
                 # update the queue tree
                 self.queue_clicked(self.queue_tree)
         dprint("TERMINAL: leaving kill()")
-        return
+        return True
+
+    def confirm_delete(self, widget = None, *event):
+	if self.task_completed:
+	    return False
+	err = _("Confirm: Kill the Running Process")
+        dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
+                                gtk.MESSAGE_QUESTION,
+                                gtk.BUTTONS_YES_NO, err);
+        result = dialog.run()
+        dialog.destroy()
+        if result != gtk.RESPONSE_YES:
+	    dprint("TERMINAL: confirm_delete(); stopping delete")
+            return True
+	dprint("TERMINAL: confirm_delete(); confirmed")
+	return False
+
+	
+    def overwrite(self, num, text, tagname = None):
+        """ Overwrite text to a text buffer.  Line numbering based on
+            the process window line count is automatically added.
+            BUT -- if multiple text buffers are going to be updated,
+            always update the process buffer LAST to guarantee the
+            line numbering is correct.
+            Optionally, text formatting can be applied as well
+        """
+        #dprint("TERMINAL: overwrite() -- num= %d:" %num)
+        #dprint(self.term.current_tab)
+        line_number = self.term.buffer[TAB_PROCESS].get_line_count() 
+        iter = self.term.buffer[num].get_iter_at_line(line_number)
+        iter.set_line_offset(7)
+        end = iter.copy()
+        end.forward_line()
+        self.term.buffer[num].delete(iter, end)
+        if tagname == None:
+           self.term.buffer[num].insert(iter, text)
+        else:
+           self.term.buffer[num].insert_with_tags_by_name(iter, text, tagname)
+
+    def scroll_current_view(self):
+        """scrolls the current_tab"""
+        if self.term.current_tab != TAB_QUEUE:
+	    self.term.view[self.term.current_tab].scroll_mark_onscreen(self.term.end_mark[self.term.current_tab])
 
     def append(self, num, text, tagname = None):
         """ Append text to a text buffer.  Line numbering based on
@@ -592,13 +689,7 @@ class ProcessManager:
         else:
            self.term.buffer[num].insert_with_tags_by_name(iter, text, tagname)
         if self.term.auto_scroll[num] and num == self.term.current_tab:
-            #dprint("TERMINAL: append() -- self.term.vadjustment[], self.term.vhandler_id[]")
-            #dprint(self.term.vadjustment)
-            #dprint(self.term.vhandler_id)
-            #self.term.vadjustment[num].handler_block(self.term.vhandler_id[num])
-            #result = self.term.view[num].scroll_to_iter(self.term.buffer[num].get_end_iter(),0.0, True, 0, 0.9)
-            self.term.view[num].scroll_mark_onscreen(self.term.buffer[num].get_insert())
-            #self.term.vadjustment[num].handler_unblock(self.term.vhandler_id[num])
+	    self.scroll_current_view()
 
     def append_all(self, text, all = False, tag = None):
         """ Append text to all buffers """
@@ -624,11 +715,25 @@ class ProcessManager:
         self.reader.string_locked = True
         for char in self.reader.string:
             if char:
-                # if we find a CR without a LF, clear the buffer
+                # if we find a CR without a LF, switch to overwrite mode
                 if cr_flag:
-                   if char != '\n':
-                      self.process_buffer = ''
-                   cr_flag = False
+                    if char != '\n':
+                        tag = None
+                        if self.first_cr:
+                            #dprint("TERMINAL: self.first_cr = True")
+                            self.append(TAB_PROCESS, self.process_buffer, tag)
+                            self.first_cr = False
+                            self.overwrite_till_nl = True
+                            self.process_buffer = ''
+                        # overwrite until after a '\n' detected for this line
+                        else:
+                            #dprint("TERMINAL: self.first_cr = False")
+                            self.overwrite(TAB_PROCESS, self.process_buffer, tag)
+                            self.process_buffer = ''
+                    else:
+                        # reset for next time
+                        self.first_cr = True
+                    cr_flag = False
                 # catch portage escape sequence NOCOLOR bugs
                 if ord(char) == 27 or self.catch_seq:
                         self.catch_seq = True
@@ -693,17 +798,26 @@ class ProcessManager:
                             self.append(TAB_CAUTION, self.process_buffer)
                             self.caution_count += 1
 
-                        self.append(TAB_PROCESS, self.process_buffer, tag)
+                        if self.overwrite_till_nl:
+                            #dprint("TERMINAL: '\\n' detected in overwrite mode")
+                            self.overwrite(TAB_PROCESS, self.process_buffer, tag)
+                            self.overwrite_till_nl = False
+                        else:
+                            self.append(TAB_PROCESS, self.process_buffer, tag)
                         self.process_buffer = ''  # reset buffer
                 elif ord(char) == 13: # carriage return?
                     pass
+            elif self.process_buffer == ">>> Updating Portage cache...":
+                # print it to screen so the user knows what is happening
+                self.append(TAB_PROCESS, self.process_buffer, tag)
+                self.process_buffer = ''  # reset buffer
         self.reader.string = ""
         #dprint("TERMINAL: update() checking file input/reader finished")
         if self.file_input and not self.reader.file_input: # reading file finished
             dprint("LOG: update()... end of file input... cleaning up")
             self.term.buffer[TAB_PROCESS].set_modified(gtk.FALSE)
             self.finish_update()
-            self.set_statusbar("*** Log loading complete : %s" % self.filename)
+            self.set_statusbar(_("*** Log loading complete : %s") % self.filename)
             self.reader.f.close()
             self.file_input = False
         # unlock the string
@@ -726,13 +840,13 @@ class ProcessManager:
 
     def finish_update(self):
         if self.warning_count != 0:
-            self.append(TAB_INFO, "*** Total warnings count for merge = %d \n"\
+            self.append(TAB_INFO, _("*** Total warnings count for merge = %d \n")\
                         %self.warning_count, 'note')
             if not self.term.tab_showing[TAB_INFO]:
                 self.show_tab(TAB_INFO)
                 self.term.buffer[TAB_INFO].set_modified(gtk.TRUE)
         if self.caution_count != 0:
-            self.append(TAB_INFO, "*** Total cautions count for merge = %d \n"\
+            self.append(TAB_INFO, _("*** Total cautions count for merge = %d \n")\
                         %self.caution_count, 'note')
             if not self.term.tab_showing[TAB_INFO]:
                 self.show_tab(TAB_INFO)
@@ -742,6 +856,10 @@ class ProcessManager:
     def process_done(self):
         """ Remove the finished process from the queue, and
         start the next one if there are any more to be run"""
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
+        dprint("TERMINAL: process_done; Semaphore acquired")
+        
         # reset to None, so next one starts properly
         self.reader.fd = None
         # if the last process was killed, stop until the user does something
@@ -749,6 +867,9 @@ class ProcessManager:
             # display message that process has been killed
             self.append_all(KILLED_STRING,True)
             self.set_statusbar(KILLED_STRING[:-1])
+            # Clear semaphore, we're done
+            self.Semaphore.release()
+            dprint("TERMINAL: process_done; Semaphore released")
             return
             
         # If the user did an emerge --pretend, we print out
@@ -760,16 +881,25 @@ class ProcessManager:
         self.append_all(TERMINATED_STRING,True)
         self.set_statusbar(TERMINATED_STRING[:-1])
         # try to get a callback
-        callback = self.process_list[0][3]
+        try:
+            callback = self.process_list[0][3]
+        except:
+            callback = None
         # set queue icon to done
-        iter = self.process_list[0][2]
-        # set icon according to success or failure
-        if self.Failed:
-            self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_STOP))
-        else:
-            self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_APPLY))
+        try:
+            iter = self.process_list[0][2]
+        	# set icon according to success or failure
+            if self.Failed:
+                self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_STOP))
+            else:
+                self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_APPLY))
+        except: pass
         # remove process from list
         self.process_list = self.process_list[1:]
+	self.task_completed = True
+        # We're finished, release semaphore
+        self.Semaphore.release()
+        dprint("TERMINAL: process_done; Semaphore released")
         # check for pending processes, and run them
         self.start_queue(False)
         # if there is a callback set, call it
@@ -777,30 +907,14 @@ class ProcessManager:
             callback()
         if self.term.tab_showing[TAB_QUEUE]:
             # update the queue tree
-            self.queue_clicked(self.queue_tree)
+            result = self.queue_clicked(self.queue_tree)
+
 
     def render_icon(self, icon):
         """ Render an icon for the queue tree """
         return self.queue_tree.render_icon(icon,
                     size = gtk.ICON_SIZE_MENU, detail = None)
 
-    def kill_process(self, widget):
-        """ Kill currently running process """
-        if not self.reader.process_running and not self.file_input:
-            dprint("TERMINAL: No running process to kill!")
-            return
-        self.kill()
-        if self.log_mode:
-            dprint("LOG: set statusbar -- log killed")
-            self.set_statusbar("***Log Process Killed!")
-        else:
-            # set the queue icon to killed
-            iter = self.process_list[0][2]
-            self.queue_model.set_value(iter, 0, self.render_icon(gtk.STOCK_CANCEL))
-            # set the resume buttons to sensitive
-            self.set_resume(True)
-        dprint("TERMINAL: leaving kill_process")
-        return
 
     def extract_num(self, line):
         """extracts the number of packages from the 'emerge (x of y) cat/package
@@ -826,31 +940,39 @@ class ProcessManager:
                 self.skip_first_menu.set_sensitive(gtk.FALSE)
             self.resume_menu.set_sensitive(gtk.TRUE)
             # check if there are more queue entries to process
+	    self.Semaphore.acquire()
             if len(self.process_list)> 1:
                 self.skip_queue_menu.set_sensitive(gtk.TRUE)
             else:
                 self.skip_queue_menu.set_sensitive(gtk.FALSE)
+	    self.Semaphore.release()
         else:
             self.resume_menu.set_sensitive(gtk.FALSE)
     
     def resume_normal(self, widget):
         """ Resume killed process """
+	Self.Semaphore.acquire()
         # pass the normal command along with --resume
         name, command, iter, callback = self.process_list[0]
-        command += " --resume"
+	command += " --resume"
         self._run(command, iter)
+	self.Semaphore.release()
 
     def resume_skip_first(self, widget):
         """ Resume killed process, skipping first package """
+	Self.Semaphore.acquire()
         # pass the normal command along with --resume --skipfirst
         name, command, iter, callback = self.process_list[0]
         command += " --resume --skipfirst"
         self._run(command, iter)
+	self.Semaphore.release()
 
     # skip_first needs to be true for the menu callback
     def start_queue(self, skip_first = True):
         """skips the first item in the process_list"""
         dprint("TERMINAL: start_queue()")
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
         if skip_first:
             dprint("         ==> skipping killed process")
             self.resume_available = False
@@ -860,20 +982,29 @@ class ProcessManager:
             if callback:
                 callback()
             if self.term.tab_showing[TAB_QUEUE]:
-                # update the queue tree
-                self.queue_clicked(self.queue_tree)
-                # remove process from list
+		self.Semaphore.release()
+                # update the queue tree wait for it to return, it might prevent crashes
+                result = self.queue_clicked(self.queue_tree)
+		self.Semaphore.acquire()
+		# remove process from list
                 self.process_list = self.process_list[1:]
         # check for pending processes, and run them
         #dprint(self.process_list)
         if len(self.process_list):
             dprint("TERMINAL: There are pending processes, running now... [" + \
                     self.process_list[0][0] + "]")
+	    self.task_completed = False
             self._run(self.process_list[0][1], self.process_list[0][2])
         else: # re-activate the open/save menu items
             self.save_menu.set_sensitive(gtk.TRUE)
             self.save_as_menu.set_sensitive(gtk.TRUE)
             self.open_menu.set_sensitive(gtk.TRUE)
+	# We're finished, release semaphore
+	self.Semaphore.release()
+        dprint("TERMINAL: start_queue(); finnished... returning")
+	return
+
+
 
 
     def copy_selected(self, widget):
@@ -896,6 +1027,10 @@ class ProcessManager:
         """ Switch two adjacent queue items;
             direction is either 1 [down] or -1 [up] """
         dprint("TERMINAL: Switching queue items.")
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
+        dprint("TERMINAL: queue_items_switch; Semaphore acquired")
+
         # get the selected iter
         iter = get_treeview_selection(self.queue_tree)
         # get its path
@@ -922,15 +1057,19 @@ class ProcessManager:
                           self.process_list[pos + direction][2], \
                           self.process_list[pos][3]
                     prev = self.process_list[pos + direction][0],\
-                           self.process_list[pos + direction][1],\
-                           self.process_list[pos][2],\
-                           self.process_list[pos + direction][3]
+                          self.process_list[pos + direction][1],\
+                          self.process_list[pos][2],\
+                          self.process_list[pos + direction][3]
                     self.process_list[pos] = prev
                     self.process_list[pos + direction] = sel
                     break
         else:
             dprint("TERMINAL: cannot move first or last item")
-        self.queue_clicked(self.queue_tree)
+        result = self.queue_clicked(self.queue_tree)
+
+        # We're done, release semaphore
+        self.Semaphore.release()
+        dprint("TERMINAL: queue_items_switch; Semaphore released")
 
     def move_queue_item_up(self, widget):
         """ Move selected queue item up in the queue """
@@ -942,6 +1081,10 @@ class ProcessManager:
 
     def remove_queue_item(self, widget):
         """ Remove the selected item from the queue """
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
+        dprint("TERMINAL: remove_queue_item; Semaphore acquired")
+
         # get the selected iter
         iter = get_treeview_selection(self.queue_tree)
         # find if this item is still in our process list
@@ -954,6 +1097,10 @@ class ProcessManager:
                 break
         self.queue_model.remove(iter)
 
+        # We're done, release semaphore
+        self.Semaphore.release()
+        dprint("TERMINAL: remove_queue_item; Semaphore released")
+
     def estimate_build_time(self):
         """Estimates build times based on emerge --pretend output"""
         start_iter = self.term.buffer[TAB_PROCESS].get_iter_at_mark(self.command_start)
@@ -1010,6 +1157,9 @@ class ProcessManager:
 
     def queue_clicked(self, widget):
         """Handle clicks to the queue treeview"""
+	dprint("TERMINAL: queue_clicked()")
+        # Prevent conflicts while changing process queue
+        self.Semaphore.acquire()
         # get the selected iter
         iter = get_treeview_selection(self.queue_tree)
         # get its path
@@ -1018,7 +1168,10 @@ class ProcessManager:
         except:
             dprint("TERMINAL: Couldn't get queue view treeiter path, " \
                    "there is probably nothing selected.")
-            return
+	    # We're finished, release semaphore
+	    self.Semaphore.release()
+	    dprint("TERMINAL: queue_clicked(); finnished... returning")
+            return False
         # if the item is not in the process list
         # don't make the controls sensitive and return
         name = get_treeview_selection(self.queue_tree, 1)
@@ -1034,7 +1187,10 @@ class ProcessManager:
                 self.queue_remove.set_sensitive(gtk.FALSE)
             else:
                 self.queue_remove.set_sensitive(gtk.TRUE)
-            return
+	    # We're finished, release semaphore
+	    self.Semaphore.release()
+	    dprint("TERMINAL: queue_clicked(); finnished... returning")
+	    return True
         # if we reach here it's still in the process list
         # activate the delete item
         self.queue_remove.set_sensitive(gtk.TRUE)
@@ -1053,6 +1209,10 @@ class ProcessManager:
             # enable moving the item
             self.move_up.set_sensitive(gtk.TRUE)
             self.move_down.set_sensitive(gtk.TRUE)
+	# We're finished, release semaphore
+	self.Semaphore.release()
+        dprint("TERMINAL: queue_clicked(); finnished... returning")
+	return True
 
     def set_save_buffer(self):
         """Sets the save info for the notebook tab's visible buffer"""
@@ -1070,24 +1230,24 @@ class ProcessManager:
         self.log_mode = True
         if not self.window_visible: self.show_window()
         if not self.fill_buffer(filename):
-            self.set_statusbar("*** Unknown File Loading error")
+            self.set_statusbar(_("*** Unknown File Loading error"))
             return gtk.FALSE
         else:
             self.filename = filename
-            self.set_statusbar("*** File Loading... Processing...") 
+            self.set_statusbar(_("*** File Loading... Processing...")) 
             return gtk.TRUE;
 
     def do_open(self, widget):
-        """opens the file selctor for file to open"""
+        """opens the file selector for file to open"""
         dprint("LOG: Entering do_open")
         if not self.directory:
             self.set_directory()
         try:
-            FileSel(self.title + ": Open log File").run(self.window,
+            FileSel(self.title + _(": Open log File")).run(self.window,
                                                         self.directory+"*.log",
                                                         self.open_ok_func)
         except:
-            FileSel(self.title + ": Open log File").run(None,
+            FileSel(self.title + _(": Open log File")).run(None,
                                                         self.directory+"*.log",
                                                         self.open_ok_func)
         dprint("LOG: leaving do_open")
@@ -1128,7 +1288,7 @@ class ProcessManager:
 
         if (not self.filename or filename != self.filename):
             if os.path.exists(filename):
-                err = "Ovewrite existing file '%s'?"  % filename
+                err = _("Ovewrite existing file '%s'?")  % filename
                 dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                                            gtk.MESSAGE_QUESTION,
                                            gtk.BUTTONS_YES_NO, err);
@@ -1191,11 +1351,11 @@ class ProcessManager:
         self.clear_buffer(None)
         self.warning_count = 0
         self.caution_count = 0
-        self.set_statusbar("*** Loading File : %s" % self.filename)
+        self.set_statusbar(_("*** Loading File : %s") % self.filename)
         try:
             self.reader.f = open(filename, "r")
         except IOError, (errnum, errmsg):
-            err = "Cannot open file '%s': %s" % (filename, errmsg)
+            err = _("Cannot open file '%s': %s") % (filename, errmsg)
             dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                                        gtk.MESSAGE_INFO,
                                        gtk.BUTTONS_OK, err);
@@ -1208,7 +1368,7 @@ class ProcessManager:
         return gtk.TRUE
 
     def save_buffer(self):
-        """save the contens of the buffer"""
+        """save the contents of the buffer"""
         dprint("LOG: Entering save_buffer")
         result = gtk.FALSE
         have_backup = gtk.FALSE
@@ -1220,7 +1380,7 @@ class ProcessManager:
             os.rename(self.filename, bak_filename)
         except (OSError, IOError), (errnum, errmsg):
             if errnum != errno.ENOENT:
-                err = "Cannot back up '%s' to '%s': %s" % (self.filename,
+                err = _("Cannot back up '%s' to '%s': %s") % (self.filename,
                                                            bak_filename,
                                                            errmsg)
                 dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
@@ -1231,7 +1391,7 @@ class ProcessManager:
                 return gtk.FALSE
 
         have_backup = gtk.TRUE
-        self.set_statusbar("*** saving file: %s" % self.filename)
+        self.set_statusbar(_("*** saving file: %s") % self.filename)
         try:
             file = open(self.filename, "w")
             # if buffer is "Process" strip line numbers
@@ -1253,7 +1413,7 @@ class ProcessManager:
             self.buffer_to_save.set_modified(gtk.FALSE)
             result = gtk.TRUE
         except IOError, (errnum, errmsg):
-            err = "Error writing to '%s': %s" % (self.filename, errmsg)
+            err = ("Error writing to '%s': %s") % (self.filename, errmsg)
             dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                                        gtk.MESSAGE_INFO,
                                        gtk.BUTTONS_OK, err);
@@ -1264,7 +1424,7 @@ class ProcessManager:
             try:
                 os.rename(bak_filename, self.filename)
             except OSError, (errnum, errmsg):
-                err = "Can't restore backup file '%s' to '%s': %s\nBackup left as '%s'" % (
+                err = _("Can't restore backup file '%s' to '%s': %s\nBackup left as '%s'") % (
                     self.filename, bak_filename, errmsg, bak_filename)
                 dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                                            gtk.MESSAGE_INFO,
@@ -1272,7 +1432,7 @@ class ProcessManager:
                 dialog.run()
                 dialog.destroy()
 
-        self.set_statusbar("*** File saved : %s" % self.filename)
+        self.set_statusbar(_("*** File saved : %s") % self.filename)
         dprint("LOG: Buffer saved, exiting")
         return result
 
@@ -1282,7 +1442,7 @@ class ProcessManager:
         self.filename = self.pretty_name()
         if buffer.get_modified():
             if save:
-                msg = "Save log to '%s'?" % self.filename
+                msg = _("Save log to '%s'?") % self.filename
                 dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                                            gtk.MESSAGE_QUESTION,
                                            gtk.BUTTONS_YES_NO, msg);
@@ -1351,7 +1511,7 @@ class terminal_notebook:
         self.buffer_types = {TAB_PROCESS:"log",
                              TAB_WARNING:"warning",
                              TAB_CAUTION:"caution",
-                             TAB_INFO:"info",
+                             TAB_INFO:"summary",
                              TAB_QUEUE:None}
         self.tab = [] #[None, None, None, None]
         self.visible_tablist = []
@@ -1360,16 +1520,10 @@ class terminal_notebook:
         self.vadjustment = [] #[None, None, None, None, None]
         self.vhandler_id = [] #[None, None, None, None, None]
         self.auto_scroll = [True, False, False, False, False]
+	self.end_mark = [] # hold the end of buffer text marks for autoscrolling
         self.get_tab_list() # initialize to default state
 
 
-    def scroll_current_view(self):
-        """scrolls the current_tab"""
-        if self.current_tab != TAB_QUEUE:
-            self.vadjustment[self.current_tab].handler_block(self.vhandler_id[self.current_tab])
-            self.view[self.current_tab].scroll_mark_onscreen(self.buffer[self.current_tab].get_insert())
-            self.vadjustment[self.current_tab].handler_unblock(self.vhandler_id[self.current_tab])
-
     def get_tab_list(self):
         """creates the current notebook tab list"""
         #tabs_showing = 0
@@ -1413,6 +1567,7 @@ class ProcessOutputReader(threading.Thread):
 
     def run(self):
         """ Watch for process output """
+        char = None
         while True:
             if self.process_running or self.file_input:
                 # get the output and pass it to self.callback()
@@ -1445,7 +1600,7 @@ class ProcessOutputReader(threading.Thread):
                     # clean up, process is terminated
                     self.process_running = False
                     while self.string != "":
-                        dprint("TERMINAL ProcessOutputReader: waiting for update to finish")
+                        #dprint("TERMINAL ProcessOutputReader: waiting for update to finish")
                         # wait for update_callback to finish
                         time.sleep(.5)
                     if self.file_input:
