@@ -25,7 +25,7 @@
 # initially set debug to false
 debug = False
 
-import os
+import os, threading
 import errno
 import string
 import sre
@@ -34,21 +34,18 @@ import datetime
 from sys import stderr
 from version import version
 from xmlmgr import XMLManager, XMLManagerError
-
-def dprint(message):
-    """Print debug message if debug is true."""
-    if debug:
-        print >>stderr, message
+from gettext import gettext as _
 
 import pygtk; pygtk.require("2.0") # make sure we have the right version
 import gtk
 import os
 import grp
-import pwd
+import pwd, cPickle
 
 # if using gnome, see if we can import it
 try:
     import gnome
+    import gnome.vfs
 except ImportError:
     # no gnome module, use the standard webbrowser module
     try:
@@ -57,15 +54,50 @@ except ImportError:
         print >>stderr, ('Module "webbrowser" not found. '
                      'You will not be able to open web pages.')
 
+def dprint(message):
+    """Print debug message if debug is true."""
+    if debug:
+        print >>stderr, message
+
+def dsave(name, item = None):
+    """saves 'item' to file 'name' if debug is true"""
+    if debug:
+        dprint("UTILS: dsave() Pickling 'item' to file: %s" %name)
+        # get home directory
+        home = pwd.getpwuid(os.getuid())[5]
+        # pickle it baby, yeah!
+        cPickle.dump(item, open(home + "/.porthole/" + name, "w"))
+        
+
 def load_web_page(name):
     """Try to load a web page in the default browser"""
-    try:
-        gnome.url_show(name)
-    except:
+    dprint("UTILS: load_web_page()")
+    browser = web_page(name)
+    browser.start()
+    return
+
+class web_page(threading.Thread):
+    """Try to load a web page in the default browser"""
+    def __init__(self, name):
+        dprint("UTILS: web_page.__init__()")
+        threading.Thread.__init__(self)
+        self.name = name
+        self.setDaemon(1)  # quit even if this thread is still running
+        
+    def run(self):
+        dprint("UTILS: web_page.run()")
+        if self.name == '' or self.name == None:
+            return
         try:
-            webbrowser.open(name)
+            gnome.url_show(self.name)
         except:
-            pass
+            dprint("Gnome failed trying to open: %s" %self.name)
+            try:
+                webbrowser.open(self.name)
+            except:
+                dprint("webbrowser failed trying to open: %s  -- giving up" %self.name)
+                pass
+        dprint("Browser call_completed for: %s" %self.name)
 
 def get_icon_for_package(package):
     """Return an icon for a package"""
@@ -158,7 +190,7 @@ class SingleButtonDialog(CommonDialog):
                                            callback, button)
         if progressbar:
             self.progbar = gtk.ProgressBar()
-            self.progbar.set_text("Loading")
+            self.progbar.set_text(_("Loading"))
             self.progbar.show()
             self.vbox.add(self.progbar)
 
@@ -169,6 +201,7 @@ class EmergeOptions:
         self.pretend = False
         self.fetch = False
         self.verbose = False
+        self.upgradeonly = False
         self.nospinner = True # currently hidden
 
     def get_string(self):
@@ -177,6 +210,7 @@ class EmergeOptions:
         if self.pretend:   opt_string += '--pretend '
         if self.fetch:     opt_string += '--fetchonly '
         if self.verbose:   opt_string += '--verbose '
+        if self.upgradeonly: opt_string += '--upgradeonly ' 
         if self.nospinner: opt_string += '--nospinner '
         return opt_string
 
@@ -221,22 +255,22 @@ class PortholePreferences:
         try:
            width = dom.getitem('/window/main/width')
         except XMLManagerError:
-           width = 500   # Default value
+           width = 200   # Default value
         try:
            height = dom.getitem('/window/main/height')
         except XMLManagerError:
-           height = 650   # Default value
+           height = 350   # Default value
         self.main = WindowPreferences(width, height)
         try:
            hpane = dom.getitem('/window/main/hpane')
         except XMLManagerError:
-           hpane = 280   # Default value
+           hpane = 180   # Default value
         self.main.hpane = hpane
         dprint("UTILS: __init__() hpane: %d" %self.main.hpane)
         try:
            vpane = dom.getitem('/window/main/vpane')
         except XMLManagerError:
-           vpane = 250   # Default value
+           vpane = 125   # Default value
         self.main.vpane = vpane
         try:
            search_desc = dom.getitem('/window/main/search_desc')
@@ -254,16 +288,16 @@ class PortholePreferences:
         try:
            width = dom.getitem('/window/process/width')
         except XMLManagerError:
-           width = 400   # Default value
+           width = 300   # Default value
         try:
            height = dom.getitem('/window/process/height')
         except XMLManagerError:
-           height = 600   # Default value
+           height = 350   # Default value
         self.process = WindowPreferences(width, height)
         try:
            width_verbose = dom.getitem('/window/process/width_verbose')
         except XMLManagerError:
-           width_verbose = 900   # Default value
+           width_verbose = 500   # Default value
         self.process.width_verbose = width_verbose
 
         # Terminal window settings
@@ -271,16 +305,16 @@ class PortholePreferences:
         try:
            width = dom.getitem('/window/terminal/width')
         except XMLManagerError:
-           width = 500   # Default value
+           width = 300   # Default value
         try:
            height = dom.getitem('/window/terminal/height')
         except XMLManagerError:
-           height = 400   # Default value
+           height = 350   # Default value
         self.terminal = WindowPreferences(width, height)
         try:
            width_verbose = dom.getitem('/window/terminal/width_verbose')
         except XMLManagerError:
-           width_verbose = 900   # Default value
+           width_verbose = 500   # Default value
         self.terminal.width_verbose = width_verbose
         
         # Formatting tags for the terminal window tabs.  
@@ -471,10 +505,31 @@ class PortholePreferences:
            self.emerge.verbose = dom.getitem('/emerge/options/verbose')
         except XMLManagerError:
            pass
+        try:
+           self.emerge.upgradeonly = dom.getitem('/emerge/options/upgradeonly')
+        except XMLManagerError:
+           pass
         try:
            self.emerge.nospinner = dom.getitem('/emerge/options/nospinner')
         except XMLManagerError:
            pass
+
+	# Misc. variables
+
+	try:
+           self.database_size = dom.getitem('/database/size')
+        except XMLManagerError:
+           self.database_size = 7000
+        try:
+           self.dbtime = dom.getitem('/database/dbtime')
+	   #dprint("UTILS: __init__(); self.dbtime =")
+	   #dprint(self.dbtime)
+        except XMLManagerError:
+           self.dbtime = 50
+        try:
+           self.dbtotals = dom.getitem('/database/dbtotals')
+        except XMLManagerError:
+           self.dbtotals = []
         
         # All prefs now loaded or defaulted
         del dom   # no longer needed, release memory
@@ -482,13 +537,14 @@ class PortholePreferences:
 
     def save(self):
         """ Save preferences """
+        dprint("UTILS: preferences save()")
         dom = XMLManager(None)
         dom.name = 'portholeprefs'
         dom.version = version
         dom.additem('/window/main/width', self.main.width)
         dom.additem('/window/main/height', self.main.height)
         dom.additem('/window/main/hpane', self.main.hpane)
-        dprint("UTILS: save() hpane: %d" %self.main.hpane)
+        #dprint("UTILS: save() hpane: %d" %self.main.hpane)
         dom.additem('/window/main/vpane', self.main.vpane)
         dom.additem('/window/main/search_desc', self.main.search_desc)
         dom.additem('/window/main/show_nag_dialog', self.main.show_nag_dialog)
@@ -512,7 +568,12 @@ class PortholePreferences:
         dom.additem('/emerge/options/pretend', self.emerge.pretend)
         dom.additem('/emerge/options/fetch', self.emerge.fetch)
         dom.additem('/emerge/options/verbose', self.emerge.verbose)
+        dom.additem('/emerge/options/upgradeonly', self.emerge.upgradeonly)
         dom.additem('/emerge/options/nospinner', self.emerge.nospinner)
+	dom.additem('/database/size', self.database_size)
+	#dprint("UTILS: save(); self.dbtime = %d" %self.dbtime)
+	dom.additem('/database/dbtime', self.dbtime)
+	dom.additem('/database/dbtotals', self.dbtotals)
         dom.save(self.__PFILE)
         del dom   # no longer needed, release memory
 
@@ -671,5 +732,22 @@ def estimate(package_name, log_file_name="/var/log/emerge.log"):
         else:
             return None          
     except:
-        raise BadLogFile, "Error reading emerge log file.  Check file " +\
-                          "permissions, or check for corrupt log file."   
+        raise BadLogFile, _("Error reading emerge log file.  Check file " +\
+                          "permissions, or check for corrupt log file.")   
+
+def pretend_check(command_string):
+    isPretend = (sre.search("--pretend", command_string) != None)
+    if not isPretend:
+        tmpcmdline = command_string.split()
+        #dprint(tmpcmdline)
+        for x in tmpcmdline:
+            if x[0:1]=="-"and x[1:2]!="-":
+                for y in x[1:]:
+                    #dprint(y)    
+                    if y == "p":
+                        #dprint("found it")
+                        isPretend = True
+    return isPretend
+
+def help_check(command_string):
+    return (sre.search("--help", command_string) != None)
